@@ -60,6 +60,7 @@ Board::Board(QWidget * parent)
     , m_cheat(false)
     , m_gravityFlag(true)
     , m_solvableFlag(false)
+    , m_showUnsolvableMessageFlag(false)
     , m_chineseStyleFlag(false)
     , m_tilesCanSlideFlag(false)
     , m_highlightedTile(-1)
@@ -114,6 +115,7 @@ void Board::loadSettings()
     // Need to load solvable before size because setSize calls newGame which
     // uses the solvable flag. Same with shuffle.
     setSolvableFlag(Prefs::solvable());
+    setShowUnsolvableMessageFlag(Prefs::showUnsolvableMessage());
     m_shuffle = Prefs::level() * 4 + 1;
     setSize(s_sizeX.at(Prefs::size()), s_sizeY.at(Prefs::size()));
     setGravityFlag(Prefs::gravity());
@@ -1346,7 +1348,9 @@ void Board::undrawConnection()
 
     PossibleMoves dummyPossibleMoves;
     // game is over?
-    if (!hint_I(dummyPossibleMoves)) {
+    if ((!pathFoundBetweenMatchingTiles(dummyPossibleMoves)
+         && m_showUnsolvableMessageFlag)
+        || (tilesLeft() == 0)) {
         m_gameClock.pause();
         Q_EMIT endOfGame();
     }
@@ -1654,19 +1658,18 @@ void Board::showHint()
 {
     undrawConnection();
 
-    if (hint_I(m_possibleMoves)) {
+    if (pathFoundBetweenMatchingTiles(m_possibleMoves)) {
         m_connection = m_possibleMoves.front().path();
         drawConnection();
     }
 }
 
-
 #ifdef DEBUGGING
 void Board::makeHintMove()
 {
     PossibleMoves possibleMoves;
 
-    if (hint_I(possibleMoves)) {
+    if (pathFoundBetweenMatchingTiles(possibleMoves)) {
         m_markX = -1;
         m_markY = -1;
         marked(TilePos(possibleMoves.front().path().front().x(), possibleMoves.front().path().front().y()));
@@ -1708,7 +1711,7 @@ int Board::lineWidth() const
     return width;
 }
 
-bool Board::hint_I(PossibleMoves & possibleMoves) const
+bool Board::pathFoundBetweenMatchingTiles(PossibleMoves & possibleMoves) const
 {
     std::array<short, Board::nTiles> done{}; // Appended {} initialises with zeroes here.
 
@@ -1754,17 +1757,17 @@ bool Board::isSolvable(bool restore)
     }
 
     PossibleMoves p;
-    while (hint_I(p)) {
+    while (pathFoundBetweenMatchingTiles(p)) {
         auto const tile1 = TilePos(p.front().path().front().x(), p.front().path().front().y());
         auto const tile2 = TilePos(p.front().path().back().x(), p.front().path().back().y());
         if (!tilesMatch(field(tile1), field(tile2))) {
-            auto errMessage = QStringLiteral("Removing unmatched tiles: (%1,%2) => %3 (%4,%5) => %6")
-                                  .arg(p.front().path().front().x())
-                                  .arg(p.front().path().front().y())
-                                  .arg(field(tile1))
-                                  .arg(p.front().path().back().x())
-                                  .arg(p.front().path().back().y())
-                                  .arg(field(tile2));
+            auto const errMessage = QStringLiteral("Removing unmatched tiles: (%1,%2) => %3 (%4,%5) => %6")
+                                        .arg(p.front().path().front().x())
+                                        .arg(p.front().path().front().y())
+                                        .arg(field(tile1))
+                                        .arg(p.front().path().back().x())
+                                        .arg(p.front().path().back().y())
+                                        .arg(field(tile2));
             qCCritical(KSHISEN_General) << errMessage;
         }
         setField(tile1, EMPTY);
@@ -1797,6 +1800,19 @@ void Board::setSolvableFlag(bool enabled)
     }
 }
 
+bool Board::showUnsolvableMessageFlag() const
+{
+    return m_showUnsolvableMessageFlag;
+}
+
+void Board::setShowUnsolvableMessageFlag(bool enabled)
+{
+    if (m_showUnsolvableMessageFlag == enabled) {
+        return;
+    }
+    m_showUnsolvableMessageFlag = enabled;
+}
+
 bool Board::gravityFlag() const
 {
     return m_gravityFlag;
